@@ -12,14 +12,19 @@
 -----------------------------------------------------------------------------
 
 module Language.Preprocessor.Cpphs
-  ( runCpphs, cppIfdef, macroPass, CpphsOptions(..), BoolOptions(..)
+  ( runCpphs, runCpphsReturningSymTab
+  , cppIfdef
+  , macroPass, macroPassReturningSymTab
+  , CpphsOptions(..), BoolOptions(..)
   , parseOptions, defaultCpphsOptions, defaultBoolOptions
   , module Language.Preprocessor.Cpphs.Position
   ) where
 
 import Language.Preprocessor.Cpphs.CppIfdef(cppIfdef)
-import Language.Preprocessor.Cpphs.MacroPass(macroPass)
-import Language.Preprocessor.Cpphs.RunCpphs(runCpphs)
+import Language.Preprocessor.Cpphs.MacroPass(macroPass
+                                            ,macroPassReturningSymTab)
+import Language.Preprocessor.Cpphs.RunCpphs(runCpphs
+                                           ,runCpphsReturningSymTab)
 import Language.Preprocessor.Cpphs.Options
        (CpphsOptions(..), BoolOptions(..), parseOptions
        ,defaultCpphsOptions,defaultBoolOptions)
